'use strict';

const chai = require('chai');
const mocha = require('mocha');
const should = require('should');

const Schema = require('../index').Schema;

describe('Schema', function() {
    this.timeout(10 * 1000);

    /* -------------------------------------------------------------------------
     * Before
     * ---------------------------------------------------------------------- */

    before(function() {
    });

    /* -------------------------------------------------------------------------
     * Create
     * ---------------------------------------------------------------------- */

    describe('Get', () => {
        let schema = null;

        it('create', () => {
            schema = new Schema({
                name: {
                    type: Schema.type.STRING,
                },
                cats: {
                    type: Schema.type.INTEGER,
                },
                age: {
                    type: Schema.type.INTEGER,
                    defaultValue: 30,
                },
                composed: {
                    type: Schema.type.STRING,
                    get () {
                        return this.name + ' has ' + this.age + ' years old';
                    }
                },
                composedValue: {
                    type: Schema.type.STRING,
                    defaultValue: 'hi',
                    get (value) {
                        return this.name + ' says ' + value;
                    }
                }

            }, {
                name: 'Wiliam',
            });
        });

        it('get values', () => {
            (schema.get('name')).should.be.equal('Wiliam');
            (schema.name).should.be.equal('Wiliam');
        });

        it('get default null', () => {
            should(schema.cats).be.null();
        });

        it('get default value', () => {
            (schema.age).should.be.equal(30);
        });

        it('get composed field', () => {
            should(schema.composed).be.null();
            should(schema.get('composed')).be.equal('Wiliam has 30 years old');
        });

        it('get composed value field', () => {
            should(schema.composedValue).be.equal('hi');
            should(schema.get('composedValue')).be.equal('Wiliam says hi');
        });

        it('get all values', () => {
            let values = schema.getValues();
            let rawValues = schema.getValues(true);

            (values.composedValue).should.be.equal('Wiliam says hi');
            (rawValues.composedValue).should.be.equal('hi');
        });
    });

    describe('Set', () => {
        let schema = null;

        it('create', () => {
            schema = new Schema({
                name: {
                    type: Schema.type.STRING,
                    allowNull: false,
                    validate: {
                        len: [2, 10],
                    }
                },
                age: {
                    type: Schema.type.STRING,
                    set (value) {
                        return value + ' years';
                    }
                }
            });
        });

        it('set a value', () => {
            schema.name = 'Wiliam';
            (schema.name).should.equal('Wiliam');

            schema.set('name', 'Marta');
            (schema.name).should.equal('Marta');
        });

        it('set a setter value', () => {
            schema.age = 30;
            (schema.age).should.equal(30);

            schema.set('age', 30);

            (schema.age).should.equal('30 years');
            (schema.get('age')).should.equal('30 years');
        });
    });

    describe('Schema getters and setters', () => {
        let schema = null;

        it('create', () => {
            schema = new Schema({
                name: {
                    type: Schema.type.STRING,
                },
                cats: {
                    type: Schema.type.INTEGER,
                    get () {
                        return this.cats + ' cats';
                    }
                },
                age: {
                    type: Schema.type.INTEGER,
                },

            }, {
                name: 'Wiliam',
                cats: 2,
                age: 30,
            }, {
                getterMethods: {
                    composed() {
                        return this.name + ' has ' + this.age + ' years and ' + this.get('cats');
                    }
                },
                setterMethods: {
                    composedSet(value) {
                        this.name = value.name;
                        this.age = value.age;
                        this.set('cats', value.cats);
                    }
                }
            });
        });

        it('get using getter', () => {
            should(schema.get('composed')).equal('Wiliam has 30 years and 2 cats');
            should(schema.composed()).equal('Wiliam has 30 years and 2 cats');
        });

        it('set using setter', () => {
            schema.set('composedSet', {
                name: 'Marta',
                age: 25,
                cats: 5,
            });

            should(schema.name).equal('Marta');
            should(schema.age).equal(25);
            should(schema.cats).equal(5);

            schema.composedSet({
                name: 'Jordi',
                age: 38,
                cats: 1,
            });

            should(schema.name).equal('Jordi');
            should(schema.age).equal(38);
            should(schema.cats).equal(1);
        });
    });

    describe('Validate', () => {
        let schema = null;

        it('create', () => {
            schema = new Schema({
                name: {
                    type: Schema.type.STRING,
                    validate: {
                        notNull: true,
                    }
                },
                cats: {
                    type: Schema.type.INTEGER,
                    validate: {
                        min: {
                            args: 1,
                            msg: 'You must have at least 1 cat',
                        },
                        max: 4,
                    }
                },
                age: {
                    type: Schema.type.INTEGER,
                    allowNull: true,
                    validate: {
                        min: 1,
                        max: 110,
                    }
                },
            }, {}, {
                validate: {
                    catsAndAge() {
                        return this.cats + this.age < 10;
                    },
                    catsVerbose() {
                        if(this.cats == 1) {
                            throw new Error('Uh!!! Only one cat??');
                        }
                    }
                }
            });
        });

        it('validate name', () => {
            return schema
                .validate()
                .should.be.rejectedWith({
                    message: 'validation failed for: notNull'
                });
        });

        it('validate cats', () => {
            schema.name = 'Wiliam';

            return schema
                .validate()
                .should.be.rejectedWith({
                    message: 'You must have at least 1 cat'
                });
        });

        it('validate max cats', () => {
            schema.cats = 10;

            return schema
                .validate()
                .should.be.rejectedWith({
                    message: 'validation failed for: max'
                });
        });

        it('validate ignore age with allowNull', () => {
            schema.cats = 3;

            return schema
                .validate()
                .should.be.fulfilled();
        });

        it('validate age', () => {
            schema.age = 200;

            return schema
                .validate()
                .should.be.rejectedWith({
                    message: 'validation failed for: max'
                });
        });

        it('validate model', () => {
            schema.age = 5;
            schema.cats = 1;

            return schema
                .validate()
                .should.be.rejectedWith({
                    message: 'Uh!!! Only one cat??'
                });
        });
    });


});